"""Tests for the forms of the aps_bom app."""
import os

from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase
from django.utils.timezone import now

from ..forms import BOMUploadForm, CBOMUploadForm
from ..models import BOM, BOMItem, CBOM, CBOMItem, EPN
from .factories import (
    CompanyFactory,
    EPNFactory,
    IPNFactory,
    UnitFactory,
)


class BOMUploadFormTestCase(TestCase):
    """TestCase for the BOMUploadForm form class."""
    longMessage = True

    def setUp(self):
        self.unit = UnitFactory(code='pcs', description='pieces')

        self.ipn = IPNFactory()
        IPNFactory(code='7900.1500')

        self.csv_file = open(os.path.join(
            settings.APP_ROOT, 'tests/files/BOM.csv'))

        self.data = {
            'ipn': self.ipn.pk,
            'description': 'BOM description',
        }

        self.files = {
            'csv_file': SimpleUploadedFile('BOM.csv', self.csv_file.read())
        }

    def tearDown(self):
        self.csv_file.close()

    def test_form(self):
        bad_data = self.data.copy()
        bad_data.update({'ipn': ''})
        form = BOMUploadForm(data=bad_data, files=self.files)
        self.assertFalse(form.is_valid(), msg='The form should not be valid.')

        form = BOMUploadForm(data=self.data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because not all IPNs are in the'
            ' database, yet.'))

        IPNFactory(code='6030.2345')
        form = BOMUploadForm(data=self.data, files=self.files)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid once there are all EPNs in the database.'
            ' Errors: {0}'.format(form.errors)))

        form.save()
        self.assertEqual(BOM.objects.count(), 1, msg=(
            'Form save did not create the correct amount of BOMs.'))
        self.assertEqual(BOMItem.objects.count(), 2, msg=(
            'Form save did not create the correct amount of BOMItems.'))

        self.unit.delete()
        form = BOMUploadForm(data=self.data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because not all units exist.'))


class CBOMUploadFormTestCase(TestCase):
    """TestCase for the CBOMUploadForm form class."""
    longMessage = True

    def setUp(self):
        self.customer = CompanyFactory()
        self.unit = UnitFactory(code='pcs', description='pieces')

        self.epn = EPNFactory(epn='945382', company=self.customer)

        self.data = {
            'customer': self.customer.id,
            'description': 'Description for the CBOM',
            'html_link': '',
            'product': 'This product is used here',
            'version_date': now(),
        }

        self.csv_file = open(os.path.join(
            settings.APP_ROOT, 'tests/files/cBOM.csv'))

        self.files = {
            'csv_file': SimpleUploadedFile('cBOM.csv', self.csv_file.read())
        }

    def tearDown(self):
        self.csv_file.close()

    def test_form(self):
        bad_data = self.data.copy()
        bad_data.update({'description': ''})
        form = CBOMUploadForm(data=bad_data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because not all EPNs are in the'
            ' database, yet.'))

        form = CBOMUploadForm(data=self.data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because not all EPNs are in the'
            ' database, yet.'))

        epn = EPN.objects.get(epn='743629')
        form = CBOMUploadForm(data=self.data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because the EPNs don\'t all have'
            ' IPN or CPN added.'))

        epn.ipn = IPNFactory()
        epn.cpn = IPNFactory()
        epn.save()
        form = CBOMUploadForm(data=self.data, files=self.files)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid once there are all EPNs in the database.'
            ' Errors: {0}'.format(form.errors)))

        form.save()
        self.assertEqual(CBOM.objects.count(), 1, msg=(
            'Form save did not create the correct amount of CBOMs.'))
        self.assertEqual(CBOMItem.objects.count(), 2, msg=(
            'Form save did not create the correct amount of CBOMItems.'))

        self.unit.delete()
        form = CBOMUploadForm(data=self.data, files=self.files)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid, because not all units exist.'))



# flake8: noqa
# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'AdditionalText'
        db.create_table(u'aps_bom_additionaltext', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ipn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='additional_texts', to=orm['aps_bom.IPN'])),
            ('text', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal(u'aps_bom', ['AdditionalText'])

        # Adding model 'BOM'
        db.create_table(u'aps_bom_bom', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('ipn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='boms', to=orm['aps_bom.IPN'])),
        ))
        db.send_create_signal(u'aps_bom', ['BOM'])

        # Adding model 'BOMItem'
        db.create_table(u'aps_bom_bomitem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('bom', self.gf('django.db.models.fields.related.ForeignKey')(related_name='bomitems', to=orm['aps_bom.BOM'])),
            ('ipn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='bomitems', to=orm['aps_bom.IPN'])),
            ('position', self.gf('django.db.models.fields.CharField')(max_length=5)),
            ('qty', self.gf('django.db.models.fields.FloatField')()),
            ('unit', self.gf('django.db.models.fields.related.ForeignKey')(related_name='bomitems', to=orm['aps_bom.Unit'])),
        ))
        db.send_create_signal(u'aps_bom', ['BOMItem'])

        # Adding model 'CBOM'
        db.create_table(u'aps_bom_cbom', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('customer', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cboms', to=orm['aps_bom.Company'])),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('html_link', self.gf('django.db.models.fields.URLField')(max_length=256)),
            ('product', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('version_date', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal(u'aps_bom', ['CBOM'])

        # Adding model 'CBOMItem'
        db.create_table(u'aps_bom_cbomitem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('bom', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cbomitems', to=orm['aps_bom.CBOM'])),
            ('epn', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('position', self.gf('django.db.models.fields.CharField')(max_length=5)),
            ('qty', self.gf('django.db.models.fields.FloatField')()),
            ('unit', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cbomitems', to=orm['aps_bom.Unit'])),
        ))
        db.send_create_signal(u'aps_bom', ['CBOMItem'])

        # Adding model 'Company'
        db.create_table(u'aps_bom_company', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(related_name='companies', to=orm['aps_bom.Country'])),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=128)),
        ))
        db.send_create_signal(u'aps_bom', ['Company'])

        # Adding model 'Country'
        db.create_table(u'aps_bom_country', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=2)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=128)),
        ))
        db.send_create_signal(u'aps_bom', ['Country'])

        # Adding model 'EPN'
        db.create_table(u'aps_bom_epn', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('company', self.gf('django.db.models.fields.related.ForeignKey')(related_name='epns', to=orm['aps_bom.Company'])),
            ('cpn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cpns', to=orm['aps_bom.IPN'])),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('epn', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('ipn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='ipns', to=orm['aps_bom.IPN'])),
        ))
        db.send_create_signal(u'aps_bom', ['EPN'])

        # Adding model 'IPN'
        db.create_table(u'aps_bom_ipn', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('price_group', self.gf('django.db.models.fields.related.ForeignKey')(related_name='ipns', to=orm['aps_bom.PriceGroup'])),
            ('shape', self.gf('django.db.models.fields.related.ForeignKey')(related_name='ipns', to=orm['aps_bom.Shape'])),
            ('price_max', self.gf('django.db.models.fields.DecimalField')(max_digits=11, decimal_places=3)),
        ))
        db.send_create_signal(u'aps_bom', ['IPN'])

        # Adding model 'PriceGroup'
        db.create_table(u'aps_bom_pricegroup', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('add', self.gf('django.db.models.fields.DecimalField')(default='0.00000', max_digits=10, decimal_places=5)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('rate', self.gf('django.db.models.fields.DecimalField')(default='1.00000', max_digits=10, decimal_places=5)),
        ))
        db.send_create_signal(u'aps_bom', ['PriceGroup'])

        # Adding model 'PriceMarker'
        db.create_table(u'aps_bom_pricemarker', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ipn', self.gf('django.db.models.fields.related.ForeignKey')(related_name='price_markers', to=orm['aps_bom.IPN'])),
            ('price', self.gf('django.db.models.fields.DecimalField')(max_digits=10, decimal_places=5)),
            ('date', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal(u'aps_bom', ['PriceMarker'])

        # Adding model 'Shape'
        db.create_table(u'aps_bom_shape', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
        ))
        db.send_create_signal(u'aps_bom', ['Shape'])

        # Adding model 'Unit'
        db.create_table(u'aps_bom_unit', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=5)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=20)),
        ))
        db.send_create_signal(u'aps_bom', ['Unit'])


    def backwards(self, orm):
        # Deleting model 'AdditionalText'
        db.delete_table(u'aps_bom_additionaltext')

        # Deleting model 'BOM'
        db.delete_table(u'aps_bom_bom')

        # Deleting model 'BOMItem'
        db.delete_table(u'aps_bom_bomitem')

        # Deleting model 'CBOM'
        db.delete_table(u'aps_bom_cbom')

        # Deleting model 'CBOMItem'
        db.delete_table(u'aps_bom_cbomitem')

        # Deleting model 'Company'
        db.delete_table(u'aps_bom_company')

        # Deleting model 'Country'
        db.delete_table(u'aps_bom_country')

        # Deleting model 'EPN'
        db.delete_table(u'aps_bom_epn')

        # Deleting model 'IPN'
        db.delete_table(u'aps_bom_ipn')

        # Deleting model 'PriceGroup'
        db.delete_table(u'aps_bom_pricegroup')

        # Deleting model 'PriceMarker'
        db.delete_table(u'aps_bom_pricemarker')

        # Deleting model 'Shape'
        db.delete_table(u'aps_bom_shape')

        # Deleting model 'Unit'
        db.delete_table(u'aps_bom_unit')


    models = {
        u'aps_bom.additionaltext': {
            'Meta': {'object_name': 'AdditionalText'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'additional_texts'", 'to': u"orm['aps_bom.IPN']"}),
            'text': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'aps_bom.bom': {
            'Meta': {'object_name': 'BOM'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'boms'", 'to': u"orm['aps_bom.IPN']"})
        },
        u'aps_bom.bomitem': {
            'Meta': {'object_name': 'BOMItem'},
            'bom': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.BOM']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.IPN']"}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'qty': ('django.db.models.fields.FloatField', [], {}),
            'unit': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.Unit']"})
        },
        u'aps_bom.cbom': {
            'Meta': {'object_name': 'CBOM'},
            'customer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cboms'", 'to': u"orm['aps_bom.Company']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'html_link': ('django.db.models.fields.URLField', [], {'max_length': '256'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'product': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'version_date': ('django.db.models.fields.DateField', [], {})
        },
        u'aps_bom.cbomitem': {
            'Meta': {'object_name': 'CBOMItem'},
            'bom': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cbomitems'", 'to': u"orm['aps_bom.CBOM']"}),
            'epn': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'qty': ('django.db.models.fields.FloatField', [], {}),
            'unit': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cbomitems'", 'to': u"orm['aps_bom.Unit']"})
        },
        u'aps_bom.company': {
            'Meta': {'object_name': 'Company'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'companies'", 'to': u"orm['aps_bom.Country']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aps_bom.country': {
            'Meta': {'object_name': 'Country'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aps_bom.epn': {
            'Meta': {'object_name': 'EPN'},
            'company': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'epns'", 'to': u"orm['aps_bom.Company']"}),
            'cpn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cpns'", 'to': u"orm['aps_bom.IPN']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'epn': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ipns'", 'to': u"orm['aps_bom.IPN']"})
        },
        u'aps_bom.ipn': {
            'Meta': {'object_name': 'IPN'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'price_group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ipns'", 'to': u"orm['aps_bom.PriceGroup']"}),
            'price_max': ('django.db.models.fields.DecimalField', [], {'max_digits': '11', 'decimal_places': '3'}),
            'shape': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ipns'", 'to': u"orm['aps_bom.Shape']"})
        },
        u'aps_bom.pricegroup': {
            'Meta': {'object_name': 'PriceGroup'},
            'add': ('django.db.models.fields.DecimalField', [], {'default': "'0.00000'", 'max_digits': '10', 'decimal_places': '5'}),
            'code': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rate': ('django.db.models.fields.DecimalField', [], {'default': "'1.00000'", 'max_digits': '10', 'decimal_places': '5'})
        },
        u'aps_bom.pricemarker': {
            'Meta': {'object_name': 'PriceMarker'},
            'date': ('django.db.models.fields.DateField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'price_markers'", 'to': u"orm['aps_bom.IPN']"}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '10', 'decimal_places': '5'})
        },
        u'aps_bom.shape': {
            'Meta': {'object_name': 'Shape'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'aps_bom.unit': {
            'Meta': {'object_name': 'Unit'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['aps_bom']



"""Forms of the aps_bom app."""
from csv import DictReader

from django import forms
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as __

from .models import BOM, BOMItem, CBOM, CBOMItem, EPN, IPN, Unit


class BaseUploadForm(forms.ModelForm):
    csv_file = forms.FileField()

    def append_error(self, error_message):
        if '__all__' in self.errors:
            self.errors['__all__'].append(error_message)
        else:
            self.errors['__all__'] = [error_message]

    def clean(self):
        cleaned_data = super(BaseUploadForm, self).clean()
        # don't do anything with the file if there are errors already
        if any(self.errors):
            return cleaned_data

        if self.Meta.model == CBOM:
            self.company = self.cleaned_data['customer']

        # initiate the csv dictreader with the uploaded file
        csv_file = self.files.get('csv_file')
        self.reader = DictReader(csv_file)
        # update the fieldnames to match the ones on the model
        new_names = []
        for fieldname in self.reader.fieldnames:
            new_names.append(fieldname.lower())
        self.reader.fieldnames = new_names

        # iterate over the lines and clean the values
        self.clean_lines = []
        for line_dict in self.reader:
            for key, value in line_dict.iteritems():
                if key == 'consign':
                    if value == '0':
                        value = False
                    else:
                        value = True
                    line_dict[key] = value
                if key == 'ipn':
                    try:
                        ipn = IPN.objects.get(code=value)
                    except IPN.DoesNotExist:
                        this_link = (
                            '<a href="{0}" target="_blank">{0}</a>'.format(
                                reverse('admin:aps_bom_ipn_add')))
                        self.append_error(__(
                            'The ipn "{0}" does not exist.'
                            ' Please create it first. {1}'.format(
                                value, this_link)))
                    else:
                        line_dict[key] = ipn
                if key == 'unit':
                    try:
                        unit = Unit.objects.get(code=value)
                    except Unit.DoesNotExist:
                        this_link = (
                            '<a href="{0}" target="_blank">{0}</a>'.format(
                                reverse('admin:aps_bom_unit_add')))
                        self.append_error(__(
                            'The unit "{0}" does not exist.'
                            ' Please create it first. {1}'.format(
                                value, this_link)))
                    else:
                        line_dict[key] = unit
                if key == 'epn':
                    try:
                        epn = EPN.objects.get(epn=value, company=self.company)
                    except EPN.DoesNotExist:
                        epn = EPN.objects.create(
                            description=line_dict.get('description'),
                            epn=value, company=self.company)
                        this_link = (
                            '<a href="{0}" target="_blank">{0}</a>'.format(
                                reverse('admin:aps_bom_epn_change',
                                        args=(epn.id, ))))
                        self.append_error(__(
                            'The EPN "{0}" does not exist.'
                            ' Please visit {1} to update it and then'
                            ' re-upload the file.'.format(
                                value, this_link)))
                    else:
                        if epn.ipn is None or epn.cpn is None:
                            this_link = (
                                '<a href="{0}" target="_blank">{0}</a>'.format(
                                    reverse('admin:aps_bom_epn_change',
                                            args=(epn.id, ))))
                            self.append_error(__(
                                'The EPN "{0}" does not have all the'
                                ' required data.'
                                ' Please visit {1} to update it and then'
                                ' re-upload the file.'.format(
                                    value, this_link)))
                        else:
                            line_dict[key] = epn
                if key == 'shape':
                    pass
            line_dict.pop('description')
            if 'shape' in line_dict:
                line_dict.pop('shape')
            self.clean_lines.append(line_dict)
        return cleaned_data


class BOMUploadForm(BaseUploadForm):
    """Custom ModelForm, that handles the upload for BOM.csv files."""

    def __init__(self, *args, **kwargs):
        super(BOMUploadForm, self).__init__(*args, **kwargs)
        self.fields['ipn'].required = True

    def save(self):
        instance = super(BOMUploadForm, self).save()
        for bomitemdict in self.clean_lines:
            bomitemdict.update({'bom': instance})
            BOMItem.objects.create(**bomitemdict)
        return instance

    class Meta:
        model = BOM
        fields = ['description', 'ipn']


class CBOMUploadForm(BaseUploadForm):
    """Custom ModelForm, that handles the upload for cBOM.csv files."""

    def save(self):
        instance = super(CBOMUploadForm, self).save()
        for cbomitemdict in self.clean_lines:
            cbomitemdict.update({'bom': instance})
            CBOMItem.objects.create(**cbomitemdict)
        return instance

    class Meta:
        model = CBOM
        fields = [
            'customer', 'description', 'html_link', 'product', 'version_date']



# flake8: noqa
# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'BOMItem.ipn'
        db.add_column(u'aps_bom_bomitem', 'ipn',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=0, related_name='bomitems', to=orm['aps_bom.IPN']),
                      keep_default=False)

        # Adding field 'CBOMItem.epn'
        db.add_column(u'aps_bom_cbomitem', 'epn',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=0, related_name='cbomitems', to=orm['aps_bom.EPN']),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'BOMItem.ipn'
        db.delete_column(u'aps_bom_bomitem', 'ipn_id')

        # Deleting field 'CBOMItem.epn'
        db.delete_column(u'aps_bom_cbomitem', 'epn_id')


    models = {
        u'aps_bom.additionaltext': {
            'Meta': {'object_name': 'AdditionalText'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'additional_texts'", 'to': u"orm['aps_bom.IPN']"}),
            'text': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'aps_bom.bom': {
            'Meta': {'object_name': 'BOM'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'boms'", 'null': 'True', 'to': u"orm['aps_bom.IPN']"})
        },
        u'aps_bom.bomitem': {
            'Meta': {'object_name': 'BOMItem'},
            'bom': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.BOM']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.IPN']"}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'qty': ('django.db.models.fields.FloatField', [], {}),
            'unit': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bomitems'", 'to': u"orm['aps_bom.Unit']"})
        },
        u'aps_bom.cbom': {
            'Meta': {'object_name': 'CBOM'},
            'customer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cboms'", 'to': u"orm['aps_bom.Company']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'html_link': ('django.db.models.fields.URLField', [], {'max_length': '256', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'product': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'version_date': ('django.db.models.fields.DateField', [], {})
        },
        u'aps_bom.cbomitem': {
            'Meta': {'object_name': 'CBOMItem'},
            'bom': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cbomitems'", 'to': u"orm['aps_bom.CBOM']"}),
            'consign': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'epn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cbomitems'", 'to': u"orm['aps_bom.EPN']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'position': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'qty': ('django.db.models.fields.FloatField', [], {}),
            'unit': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cbomitems'", 'to': u"orm['aps_bom.Unit']"})
        },
        u'aps_bom.company': {
            'Meta': {'object_name': 'Company'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'companies'", 'to': u"orm['aps_bom.Country']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aps_bom.country': {
            'Meta': {'object_name': 'Country'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aps_bom.epn': {
            'Meta': {'object_name': 'EPN'},
            'company': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'epns'", 'to': u"orm['aps_bom.Company']"}),
            'cpn': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'cpns'", 'null': 'True', 'to': u"orm['aps_bom.IPN']"}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'epn': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'ipns'", 'null': 'True', 'to': u"orm['aps_bom.IPN']"})
        },
        u'aps_bom.ipn': {
            'Meta': {'object_name': 'IPN'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'price_group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ipns'", 'to': u"orm['aps_bom.PriceGroup']"}),
            'price_max': ('django.db.models.fields.DecimalField', [], {'max_digits': '11', 'decimal_places': '3'}),
            'shape': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ipns'", 'to': u"orm['aps_bom.Shape']"})
        },
        u'aps_bom.pricegroup': {
            'Meta': {'object_name': 'PriceGroup'},
            'add': ('django.db.models.fields.DecimalField', [], {'default': "'0.00000'", 'max_digits': '10', 'decimal_places': '5'}),
            'code': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rate': ('django.db.models.fields.DecimalField', [], {'default': "'1.00000'", 'max_digits': '10', 'decimal_places': '5'})
        },
        u'aps_bom.pricemarker': {
            'Meta': {'object_name': 'PriceMarker'},
            'date': ('django.db.models.fields.DateField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'price_markers'", 'to': u"orm['aps_bom.IPN']"}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '10', 'decimal_places': '5'})
        },
        u'aps_bom.shape': {
            'Meta': {'object_name': 'Shape'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'aps_bom.unit': {
            'Meta': {'object_name': 'Unit'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['aps_bom']



"""Admin classes for the aps_bom app."""
from django.contrib import admin

from . import models


class AdditionalTextAdmin(admin.ModelAdmin):
    list_display = ['ipn', 'text']
    search_fields = ['ipn__code', 'text']


class BOMItemInline(admin.TabularInline):
    model = models.BOMItem


class BOMAdmin(admin.ModelAdmin):
    list_display = ['ipn', 'description']
    search_fields = ['ipn__code', 'description']
    inlines = [BOMItemInline, ]


class BOMItemAdmin(admin.ModelAdmin):
    list_display = ['ipn', 'bom', 'qty', 'unit', 'position']
    search_fields = ['ipn__code', 'bom__description']


class CBOMItemInline(admin.TabularInline):
    model = models.CBOMItem


class CBOMAdmin(admin.ModelAdmin):
    list_display = ['customer', 'description', 'html_link', 'product',
                    'version_date']
    search_fields = ['customer__description', 'description', 'product']
    inlines = [CBOMItemInline, ]


class CBOMItemAdmin(admin.ModelAdmin):
    list_display = ['bom', 'epn', 'qty', 'unit', 'position']
    search_fields = ['bom__description', 'bom__product', 'epn']


class CompanyAdmin(admin.ModelAdmin):
    list_display = ['code', 'description', 'country']
    search_fields = ['country', 'code', 'description']
    list_fiter = ['country', ]


class CountryAdmin(admin.ModelAdmin):
    list_display = ['code', 'description']
    search_fields = ['description']


class EPNAdmin(admin.ModelAdmin):
    list_display = ['company', 'cpn', 'epn', 'ipn', 'description']
    search_fields = ['company__description', 'cpn__code', 'epn', 'ipn__code'
                     'description']


class IPNAdmin(admin.ModelAdmin):
    list_display = ['code', 'name', 'price_group', 'shape', 'price_max']
    search_fields = ['code', 'name']
    list_filter = ['price_group']


class PriceGroupAdmin(admin.ModelAdmin):
    list_display = ['code', 'add', 'rate']


class PriceMarkerAdmin(admin.ModelAdmin):
    list_display = ['ipn', 'price', 'date']
    search_fields = ['ipn__code']


class ShapeAdmin(admin.ModelAdmin):
    list_display = ['code', 'name']
    search_fields = ['code', 'name']


class UnitAdmin(admin.ModelAdmin):
    list_display = ['code', 'description']
    search_fields = ['code', 'description']


admin.site.register(models.AdditionalText, AdditionalTextAdmin)
admin.site.register(models.BOM, BOMAdmin)
admin.site.register(models.BOMItem, BOMItemAdmin)
admin.site.register(models.CBOM, CBOMAdmin)
admin.site.register(models.CBOMItem, CBOMItemAdmin)
admin.site.register(models.Company, CompanyAdmin)
admin.site.register(models.Country, CountryAdmin)
admin.site.register(models.EPN, EPNAdmin)
admin.site.register(models.IPN, IPNAdmin)
admin.site.register(models.PriceGroup, PriceGroupAdmin)
admin.site.register(models.PriceMarker, PriceMarkerAdmin)
admin.site.register(models.Shape, ShapeAdmin)
admin.site.register(models.Unit, UnitAdmin)



"""Views for the aps_bom app."""
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import Http404
from django.views.generic import CreateView, TemplateView
from django.utils.decorators import method_decorator

from . import settings
from .forms import BOMUploadForm, CBOMUploadForm
from .models import CBOM


class BOMDownloadView(TemplateView):
    """
    View that displays a CBOM translated into BOM and allows downloading the
    BOMItem list as BOM.csv.

    """
    template_name = 'aps_bom/bom_download.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        try:
            self.cbom = CBOM.objects.get(pk=kwargs.get('cbom_pk'))
        except CBOM.DoesNotExist:
            raise Http404
        return super(BOMDownloadView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(BOMDownloadView, self).get_context_data(**kwargs)
        ctx.update({
            'object': self.cbom,
            'bom': self.cbom.get_bom(),
            'bom_items': self.cbom.get_bom_items(),
            'csv_file': self.cbom.get_bom_csv_file(),
        })
        return ctx


class BOMUploadView(CreateView):
    """
    Lets a user upload a BOM.csv file and from it create a new BOM instance.

    """
    form_class = BOMUploadForm
    template_name = 'aps_bom/bom_upload.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(BOMUploadView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(BOMUploadView, self).get_form_kwargs()
        kwargs.update({'files': self.request.FILES})
        return kwargs

    def get_success_url(self):
        return settings.BOM_UPLOAD_SUCCESS_URL


class CBOMUploadView(CreateView):
    """
    Lets a user upload a cBOM.csv file and from it create a new CBOM instance.

    """
    form_class = CBOMUploadForm
    template_name = 'aps_bom/cbom_upload.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(CBOMUploadView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(CBOMUploadView, self).get_form_kwargs()
        kwargs.update({'files': self.request.FILES})
        return kwargs

    def get_success_url(self):
        return reverse('aps_bom_bom_download', kwargs={
            'cbom_pk': self.object.pk})



"""URLs for the aps_bom app."""
from django.conf.urls import patterns, url

from .views import BOMDownloadView, BOMUploadView, CBOMUploadView


urlpatterns = patterns(
    '',
    url(r'^cbom/upload/$', CBOMUploadView.as_view(),
        name='aps_bom_cbom_upload'),
    url(r'^bom/download/(?P<cbom_pk>\d+)/$', BOMDownloadView.as_view(),
        name='aps_bom_bom_download'),
    url(r'^bom/upload/$', BOMUploadView.as_view(),
        name='aps_bom_bom_upload'),
)
